package com.m2pool.chat.interceptor;


import com.m2pool.chat.entity.StompPrincipal;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.lang.Nullable;
import org.springframework.messaging.Message;
import org.springframework.messaging.MessageChannel;
import org.springframework.messaging.simp.SimpMessageHeaderAccessor;
import org.springframework.messaging.simp.stomp.StompCommand;
import org.springframework.messaging.simp.stomp.StompHeaderAccessor;
import org.springframework.messaging.support.ChannelInterceptor;
import org.springframework.messaging.support.MessageHeaderAccessor;

import java.util.ArrayList;
import java.util.Map;

/**
 * @ClassName WebsocketChannelInterceptor
 * @Description websocket channel 通道拦截器   适用于前端@stomp/stompjs 实现.
 * @Author yyb
 * @Date 2025/4/10 15:44
 */
public class WebsocketChannelInterceptor implements ChannelInterceptor {
    private static final Logger LOGGER = LoggerFactory.getLogger(WebsocketChannelInterceptor.class);

    /**
     * websocket channel 通道拦截器
     * @param message
     * @param channel
     * @return
     */
    @Override
    public Message<?> preSend(Message<?> message, MessageChannel channel) {
        //获取链接建立时的请求头信息
        StompHeaderAccessor accessor = StompHeaderAccessor.wrap(message);
        if (accessor.getCommand() == StompCommand.CONNECT) {
            LOGGER.info("------------收到websocket的连接消息");
            //获取channel 中的请求头信息
            StompHeaderAccessor mha = MessageHeaderAccessor.getAccessor(message, StompHeaderAccessor.class);
            Object raw = message.getHeaders().get(SimpMessageHeaderAccessor.NATIVE_HEADERS);
            System.out.println("raw:"+raw);
            if (raw instanceof Map) {
                Object userInfo = ((Map) raw).get("email");
                if(userInfo instanceof ArrayList){
                    mha.setUser(new StompPrincipal((String) ((ArrayList) userInfo).get(0)));
                }
            }
        }
        if (accessor.getCommand() == StompCommand.SEND) {
            LOGGER.info("------------websocket send message");
        }

        if (accessor.getCommand() == StompCommand.SUBSCRIBE) {
            LOGGER.info("------------websocket subscribe message");
        }

        if (accessor.getCommand() == StompCommand.UNSUBSCRIBE) {
            LOGGER.info("------------websocket unsubscribe");
        }

        if (accessor.getCommand() == StompCommand.DISCONNECT){
            LOGGER.info("------------websocket disconnect");
        }
        return message;
    }


    @Override
    public void postSend(Message<?> message, MessageChannel channel, boolean sent) {
        LOGGER.info("------------WebsocketChannelInterceptor-postSend");
    }

    @Override
    public void afterSendCompletion(Message<?> message, MessageChannel channel, boolean sent, @Nullable Exception ex) {
        LOGGER.info("-----------WebsocketChannelInterceptor-afterSendCompletion");
    }

    @Override
    public boolean preReceive(MessageChannel channel) {
        LOGGER.info("----------WebsocketChannelInterceptor-preReceive");
        return true;
    }

    @Override
    public Message<?> postReceive(Message<?> message, MessageChannel channel) {
        LOGGER.info("----------WebsocketChannelInterceptor-postReceive");
        return message;
    }

    @Override
    public void afterReceiveCompletion(@Nullable Message<?> message, MessageChannel channel, @Nullable Exception ex) {
        LOGGER.info("----------WebsocketChannelInterceptor-afterReceiveCompletion");
    }
}
